/*
 * shohaku
 * Copyright (C) 2005  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.configutils;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import shohaku.composer.Composer;
import shohaku.composer.CompositeException;
import shohaku.composer.CompositeRule;
import shohaku.composer.Node;
import shohaku.core.beans.BindMethod;

/**
 * メソッドへの対応付けを行う <code>BindMethod</code> をXMLファイルから生成し、識別子でマッピングしてする機能を提供します。
 */
public class BindMethodFactory implements ConfigurationResourcesLoader {

    /* リソースデータを格納します。 */
    private final Map lookup = new Hashtable();

    /**
     * デフォルトコンストラクタ。
     */
    public BindMethodFactory() {
        //no op
    }

    /**
     * デフォルトの構成ルールを用いて、入力ストリームから構成リソースを読み込みます。
     * 
     * @param inStream
     *            構成リソースの入力ストリーム
     * @throws CompositeException
     *             構成情報例外
     */
    public void load(InputStream inStream) throws CompositeException {
        load(inStream, getDefaultComposeRule());
    }

    /**
     * 指定された構成ルールを用いて、入力ストリームから構成リソースを読み込みます。
     * 
     * @param inStream
     *            構成リソースの入力ストリーム
     * @param rule
     *            構成ルール
     * @throws CompositeException
     *             構成情報例外
     */
    public void load(InputStream inStream, CompositeRule rule) throws CompositeException {
        //XMLの解析
        Composer composer = new Composer();
        composer.parse(rule, inStream);
        Node root = composer.getRoot();
        initValues(root);
    }

    /* リソースの初期化および登録します。 */
    private void initValues(Node root) {
        Map m = new HashMap();
        initValues(root, m);
        lookup.putAll(m);
    }

    /* 再起的にリソースを登録します。 */
    private void initValues(Node parent, Map m) {
        for (Iterator i = parent.getNodeContext().elementIterator("bind-method-mapping"); i.hasNext();) {
            Node node = (Node) i.next();
            String groupId = node.getAttribute("groupId");
            initValue(node, groupId + ':', m);
        }
    }

    /* リソースを評価して登録します。 */
    private void initValue(Node parent, String groupId, Map m) {
        for (Iterator i = parent.getNodeContext().elementIterator("bind-method"); i.hasNext();) {
            Node node = (Node) i.next();
            String id = node.getAttribute("id");
            m.put(groupId + id, node.getNodeValue());
        }
    }

    /**
     * 指定された識別子の拘束されたメソッド情報を取得します。
     * 
     * @param id
     *            識別子
     * @return 指定された識別子の拘束されたメソッド情報
     */
    public BindMethod getBindMethod(String id) {
        if (id == null) {
            throw new NullPointerException("id as null");
        }
        return (BindMethod) lookup.get(id);
    }

    /*
     * static
     */

    /**
     * デフォルトの構成ルールを返却します。
     * 
     * @return デフォルトの構成ルール
     */
    public static CompositeRule getDefaultComposeRule() {
        return ConfigUtils.getDefaultComposeRule(BindMethodFactory.class);
    }
}
