/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.configutils;

import java.io.InputStream;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;

import shohaku.composer.CompositeException;
import shohaku.composer.CompositeRule;
import shohaku.composer.helpers.CompositeRuleFactory;
import shohaku.core.collections.Entry;
import shohaku.core.collections.EntryUtils;
import shohaku.core.lang.NoSuchResourceException;
import shohaku.core.lang.ResourceLoader;

/**
 * コンフィギュレーション情報を生成するユーティリティ機能を提供します。
 */
public class ConfigUtils {

    /*
     * Helper
     */

    /* 指定されたクラスローダを使用して、引数が示すリソースの入力ストリームを取得します。 */
    private static InputStream getResourceAsStream(String path, ClassLoader loader) throws NoSuchResourceException {
        return ResourceLoader.getResourceAsStream(path, loader);
    }

    ////////////////////////////////////////////////////////////////
    // XMLProperties
    ////////////////////////////////////////////////////////////////

    /* 構成ルールをキャシュします。 */
    private static final Map xmlPropertiesCache = new HashMap();

    /**
     * リソースへのクラスパスからXMLプロパティの構成リソースを生成します。
     * 
     * @param path
     *            リソースへのクラスパス
     * @return XMLプロパティラッパー
     * @throws NoSuchResourceException
     *             リソースが見つからなかった場合
     */
    public static ConfigurationResources getXMLProperties(String path) throws NoSuchResourceException {
        return getXMLProperties(path, ConfigUtils.class.getClassLoader());
    }

    /**
     * リソースへのクラスパスからXMLプロパティの構成リソースを生成します。
     * 
     * @param path
     *            リソースへのクラスパス
     * @param loader
     *            構成リソースを生成に使用するクラスローダ
     * @return XMLプロパティラッパー
     * @throws NoSuchResourceException
     *             リソースが見つからなかった場合
     */
    public static ConfigurationResources getXMLProperties(String path, ClassLoader loader)
            throws NoSuchResourceException {
        synchronized (xmlPropertiesCache) {
            ConfigurationResources resources = (ConfigurationResources) xmlPropertiesCache.get(path);
            if (resources == null) {
                resources = loadXMLProperties(path, loader);
                xmlPropertiesCache.put(path, resources);
            }
            return resources;
        }
    }

    /* XMLプロパティの読み取りを実行します。 */
    private static ConfigurationResources loadXMLProperties(String path, ClassLoader loader)
            throws NoSuchResourceException {
        InputStream inStream = getResourceAsStream(path, loader);
        XMLProperties resources = new XMLProperties();
        resources.setClassLoader(loader);
        resources.load(inStream);
        return resources;
    }

    ////////////////////////////////////////////////////////////////
    // XMLResourceBundle
    ////////////////////////////////////////////////////////////////

    /**
     * 指定された基底名、デフォルトのロケール、および呼び出し側のクラスローダを使用してXMLリソースバンドルを取得します。
     * 
     * @param baseName
     *            基底名
     * @return 指定された基底名とデフォルトのロケールのXMLリソースバンドル
     * @throws MissingResourceException
     *             指定された基底名のリソースバンドルが見つからない場合
     */
    public static ConfigurationResources getBundle(String baseName) {
        return XMLResourceBundle.getBundle(baseName);
    }

    /**
     * 指定された基底名、クラスローダ、およびデフォルトのロケールを使用してXMLリソースバンドルを取得します。.
     * 
     * @param baseName
     *            基底名
     * @param loader
     *            リソースのロード元のクラスローダ
     * @return 指定された基底名とロケールのXMLリソースバンドル
     * @throws MissingResourceException
     *             指定された基底名のリソースバンドルが見つからない場合
     */
    public static ConfigurationResources getBundle(String baseName, ClassLoader loader) {
        return XMLResourceBundle.getBundle(baseName, loader);
    }

    /**
     * 指定された基底名、ロケール、および呼び出し側のクラスローダを使用してXMLリソースバンドルを取得します。
     * 
     * @param baseName
     *            基底名
     * @param locale
     *            ロケール
     * @return 指定された基底名とロケールのXMLリソースバンドル
     * @throws MissingResourceException
     *             指定された基底名のリソースバンドルが見つからない場合
     */
    public static ConfigurationResources getBundle(String baseName, Locale locale) {
        return XMLResourceBundle.getBundle(baseName, locale);
    }

    /**
     * 指定された基底名、ロケール、クラスローダを使用してXMLリソースバンドルを取得します。
     * 
     * @param baseName
     *            基底名
     * @param locale
     *            ロケール
     * @param loader
     *            リソースのロード元のクラスローダ
     * @return 指定された基底名とロケールのXMLリソースバンドル
     * @throws MissingResourceException
     *             指定された基底名のリソースバンドルが見つからない場合
     */
    public static ConfigurationResources getBundle(String baseName, Locale locale, ClassLoader loader) {
        return XMLResourceBundle.getBundle(baseName, locale, loader);
    }

    ////////////////////////////////////////////////////////////////
    // ClassBindConfiguration
    ////////////////////////////////////////////////////////////////

    /* クラスをキーとして ClassBindConfiguration をキャシュします。 */
    private static final Map classBindConfigurationCache = new IdentityHashMap();

    /**
     * 
     * @param key
     * @return
     * @throws NoSuchResourceException
     */
    public static ConfigurationResources getClassBindConfiguration(Class key) throws NoSuchResourceException {
        return getClassBindConfiguration(key, key.getClassLoader());
    }

    /**
     * @param key
     * @param loader
     * @return
     * @throws NoSuchResourceException
     */
    public static ConfigurationResources getClassBindConfiguration(Class key, ClassLoader loader)
            throws NoSuchResourceException {
        synchronized (classBindConfigurationCache) {
            ConfigurationResources resources = (ConfigurationResources) classBindConfigurationCache.get(key);
            if (resources == null) {
                resources = loadClassBindConfiguration(key, loader);
                classBindConfigurationCache.put(key, resources);
            }
            return resources;
        }
    }

    /* ClassBindConfiguration の読み取りを実行します。 */
    private static ConfigurationResources loadClassBindConfiguration(Class key, ClassLoader loader)
            throws NoSuchResourceException {
        String resource = key.getName().replace('.', '/') + "-config.xml";
        ClassBindConfiguration classBindConfig = new ClassBindConfiguration();
        InputStream inStream = ResourceLoader.getResourceAsStream(resource, loader);
        classBindConfig.load(inStream);
        return classBindConfig;
    }

    ////////////////////////////////////////////////////////////////
    // ClassConfiguration
    ////////////////////////////////////////////////////////////////

    /* クラスをキーとして ClassConfiguration をキャシュします。 */
    private static final Map classConfigurationCache = new HashMap();

    /**
     * @param path
     * @param key
     * @return
     * @throws NoSuchResourceException
     */
    public static ConfigurationResources getClassMappingConfiguration(String path, Class key)
            throws NoSuchResourceException {
        return getClassMappingConfiguration(path, key, key.getClassLoader());
    }

    /**
     * @param path
     * @param key
     * @param loader
     * @return
     * @throws NoSuchResourceException
     */
    public static ConfigurationResources getClassMappingConfiguration(String path, Class key, ClassLoader loader)
            throws NoSuchResourceException {
        synchronized (classConfigurationCache) {
            Entry entry = EntryUtils.arrayEntry(new Object[] { path, key.getName() });
            ConfigurationResources resources = (ConfigurationResources) classConfigurationCache.get(entry);
            if (resources == null) {
                loadClassMappingConfiguration(path, loader);
                resources = (ConfigurationResources) classConfigurationCache.get(entry);
            }
            return resources;
        }
    }

    /* ClassConfiguration の読み取りを実行します。 */
    private static void loadClassMappingConfiguration(String path, ClassLoader loader) throws NoSuchResourceException {
        InputStream inStream = ResourceLoader.getResourceAsStream(path, loader);
        ClassMappingConfiguration classConfiguration = new ClassMappingConfiguration();
        classConfiguration.load(inStream);
        synchronized (classConfigurationCache) {
            for (Iterator i = classConfiguration.classNameIterator(); i.hasNext();) {
                String className = (String) i.next();
                Entry entry = EntryUtils.arrayEntry(new Object[] { path, className });
                classConfigurationCache.put(entry, classConfiguration.getResources(className));
            }
        }
    }

    /*
     * ComposeRule
     */

    /* デフォルトの構成ルールをキャシュします。 */
    private static final Map defaultComposeRuleCache = new IdentityHashMap();

    /**
     * デフォルトの構成ルールを生成して返却します。 <br>
     * カスタマイズされた構成ルールが定義されていない場合デフォルトを使用します。
     * 
     * @param c
     *            構成リソースを生成するクラス
     * @return デフォルトの構成ルール
     */
    public static CompositeRule getDefaultComposeRule(Class c) {
        return getDefaultComposeRule(c, c.getClassLoader());
    }

    /**
     * デフォルトの構成ルールを生成して返却します。 <br>
     * カスタマイズされた構成ルールが定義されていない場合デフォルトを使用します。
     * 
     * @param c
     *            構成リソースを生成するクラス
     * @param classLoader
     *            生成に使用するクラスローダ
     * @return デフォルトの構成ルール
     */
    public static CompositeRule getDefaultComposeRule(Class c, ClassLoader classLoader) {
        synchronized (defaultComposeRuleCache) {
            CompositeRule rule = (CompositeRule) defaultComposeRuleCache.get(c);
            if (rule == null) {
                rule = loadDefaultComposeRule(c, classLoader);
                defaultComposeRuleCache.put(c, rule);
            }
            return rule;
        }
    }

    /* デフォルトの構成ルールを読み取ります。 */
    private static CompositeRule loadDefaultComposeRule(Class c, ClassLoader classLoader) {
        String ruleFileName = getRuleFileBaseName(c);

        //デフォルトのファイル名
        String packagePath = c.getPackage().getName().replace('.', '/');
        String defaultRuleFile = packagePath + "/" + ruleFileName + "-default-rule.xml";

        //カスタマイズされたファイル名
        String customRuleFile = ruleFileName + "-rule.xml";

        CompositeRule rule = null;
        try {
            rule = CompositeRuleFactory.getComposeRule(customRuleFile, classLoader);
        } catch (NoSuchResourceException e) {
            //no op
        }
        if (rule == null) {
            try {
                rule = CompositeRuleFactory.getComposeRule(defaultRuleFile, classLoader);
            } catch (NoSuchResourceException e) {
                throw new CompositeException("It failed in reading of the file of the composition rule.", e);
            }
        }
        return rule;
    }

    /* 大文字を '-' + 小文字に変換する */
    private static String getRuleFileBaseName(Class c) {

        StringBuffer buf = new StringBuffer();

        String className = c.getName();
        int off = className.lastIndexOf('.');
        if (0 <= off) {
            off++;
        }
        char[] chars = className.substring(off).toCharArray();

        for (int i = 0; i < chars.length; i++) {
            char ch = chars[i];
            if (0 < i && 'A' <= ch && ch <= 'Z') {
                if ((i + 1) < chars.length) {
                    char next = chars[i + 1];
                    if ('a' <= next && next <= 'z') {
                        buf.append('-');
                    }
                } else {
                    buf.append('-');
                }
            }
            buf.append(Character.toLowerCase(ch));
        }

        return buf.toString();
    }
}
