/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.configutils;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import shohaku.core.collections.Group;

/**
 * コンフィギュレーションリソースを格納する機能を提供するラッパーオブジェクト。
 */
public class DefaultConfigurationResources extends BaseConfigurationResources {

    /**
     * デフォルトコンストラクタ。
     */
    public DefaultConfigurationResources() {
        super();
    }

    /**
     * コンフィギュレーションリソースを保管して初期化します。
     * 
     * @param resources
     *            コンフィギュレーションリソース
     */
    public DefaultConfigurationResources(ConfigurationResources resources) {
        super(resources);
    }

    /**
     * データの文字列表現(Object#toString())を取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値の文字列表現
     * @throws NullPointerException
     *             key が null の場合発生する
     */
    public String getToString(String key) {
        Object o = getObject(key);
        if (o == null) {
            return null;
        }
        return o.toString();
    }

    /**
     * データの文字列表現(Object#toString())を取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値の文字列表現
     * @throws NullPointerException
     *             key が null の場合発生する
     */
    public String getToString(String key, Object defaultValue) {
        Object o = getObject(key, defaultValue);
        if (o == null) {
            return null;
        }
        return o.toString();
    }

    /**
     * データを String 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public String getString(String key) {
        return (String) getObject(key);
    }

    /**
     * データを String 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public String getString(String key, Object defaultValue) {
        return (String) getObject(key, defaultValue);
    }

    /**
     * データを Character 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Character getCharacter(String key) {
        return (Character) getObject(key);
    }

    /**
     * データを Character 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Character getCharacter(String key, Object defaultValue) {
        return (Character) getObject(key, defaultValue);
    }

    /**
     * データを Number 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Number getNumber(String key) {
        return (Number) getObject(key);
    }

    /**
     * データを Number 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Number getNumber(String key, Object defaultValue) {
        return (Number) getObject(key, defaultValue);
    }

    /**
     * データを BigDecimal 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public BigDecimal getBigDecimal(String key) {
        return (BigDecimal) getObject(key);
    }

    /**
     * データを BigDecimal 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public BigDecimal getBigDecimal(String key, Object defaultValue) {
        return (BigDecimal) getObject(key, defaultValue);
    }

    /**
     * データを Byte 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Byte getByte(String key) {
        return (Byte) getObject(key);
    }

    /**
     * データを Byte 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Byte getByte(String key, Object defaultValue) {
        return (Byte) getObject(key, defaultValue);
    }

    /**
     * データを Short 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Short getShort(String key) {
        return (Short) getObject(key);
    }

    /**
     * データを Short 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Short getShort(String key, Object defaultValue) {
        return (Short) getObject(key, defaultValue);
    }

    /**
     * データを Integer 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Integer getInteger(String key) {
        return (Integer) getObject(key);
    }

    /**
     * データを Integer 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Integer getInteger(String key, Object defaultValue) {
        return (Integer) getObject(key, defaultValue);
    }

    /**
     * データを Long 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Long getLong(String key) {
        return (Long) getObject(key);
    }

    /**
     * データを Long 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Long getLong(String key, Object defaultValue) {
        return (Long) getObject(key, defaultValue);
    }

    /**
     * データを Float 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Float getFloat(String key) {
        return (Float) getObject(key);
    }

    /**
     * データを Float 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Float getFloat(String key, Object defaultValue) {
        return (Float) getObject(key, defaultValue);
    }

    /**
     * データを Double 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Double getDouble(String key) {
        return (Double) getObject(key);
    }

    /**
     * データを Double 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Double getDouble(String key, Object defaultValue) {
        return (Double) getObject(key, defaultValue);
    }

    /**
     * データを Boolean 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Boolean getBoolean(String key) {
        return (Boolean) getObject(key);
    }

    /**
     * データを Boolean 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Boolean getBoolean(String key, Object defaultValue) {
        return (Boolean) getObject(key, defaultValue);
    }

    /**
     * データを java.util.Date 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public java.util.Date getDate(String key) {
        return (java.util.Date) getObject(key);
    }

    /**
     * データを java.util.Date 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public java.util.Date getDate(String key, Object defaultValue) {
        return (java.util.Date) getObject(key, defaultValue);
    }

    /**
     * データを java.sql.Time 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public java.sql.Time getTime(String key) {
        return (java.sql.Time) getObject(key);
    }

    /**
     * データを java.sql.Timestamp 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public java.sql.Timestamp getTimestamp(String key, Object defaultValue) {
        return (java.sql.Timestamp) getObject(key, defaultValue);
    }

    /**
     * データを java.sql.Timestamp 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public java.sql.Timestamp getTimestamp(String key) {
        return (java.sql.Timestamp) getObject(key);
    }

    /**
     * データを java.sql.Time 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public java.sql.Time getTime(String key, Object defaultValue) {
        return (java.sql.Time) getObject(key, defaultValue);
    }

    /**
     * データを java.util.Map 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Map getMap(String key) {
        return (Map) getObject(key);
    }

    /**
     * データを java.util.Map 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Map getMap(String key, Object defaultValue) {
        return (Map) getObject(key, defaultValue);
    }

    /**
     * データを java.util.List 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public List getList(String key) {
        return (List) getObject(key);
    }

    /**
     * データを java.util.List 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public List getList(String key, Object defaultValue) {
        return (List) getObject(key, defaultValue);
    }

    /**
     * データを java.util.Set 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Set getSet(String key) {
        return (Set) getObject(key);
    }

    /**
     * データを java.util.Set 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Set getSet(String key, Object defaultValue) {
        return (Set) getObject(key, defaultValue);
    }

    /**
     * データを java.util.Collection 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Collection getCollection(String key) {
        return (Collection) getObject(key);
    }

    /**
     * データを java.util.Collection 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Collection getCollection(String key, Object defaultValue) {
        return (Collection) getObject(key, defaultValue);
    }

    /**
     * データを shohaku.core.collections.Group 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Group getGroup(String key) {
        return (Group) getObject(key);
    }

    /**
     * データを shohaku.core.collections.Group 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Group getGroup(String key, Object defaultValue) {
        return (Group) getObject(key, defaultValue);
    }

    /**
     * Type 型で定義された値を java.lang.Class 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Class getType(String key) {
        return (Class) getObject(key);
    }

    /**
     * Type 型で定義された値を java.lang.Class 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が戻り値の型にキャスト出来ない場合
     */
    public Class getType(String key, Object defaultValue) {
        return (Class) getObject(key, defaultValue);
    }

    /**
     * 配列型で定義された値を java.lang.Class 型で取得します。 <br>
     * 指定されたキーが存在しない場合 null を返却します。
     * 
     * @param key
     *            データ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が配列型でない場合に発生する
     */
    public Object getArrays(String key) {
        Object o = getObject(key);
        if (o == null) {
            return null;
        }
        if (o.getClass().isArray()) {
            return o;
        } else {
            throw new ClassCastException("Not Array Object");
        }

    }

    /**
     * 配列型で定義された値を java.lang.Class 型で取得します。 <br>
     * 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            データ名
     * @param defaultValue
     *            デフォルト値
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     * @throws ClassCastException
     *             取得値が配列型でない場合に発生する
     */
    public Object getArrays(String key, Object defaultValue) {
        Object o = getArrays(key);
        if (o == null) {
            o = defaultValue;
        }
        return o;
    }
}
