/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.configutils;

import java.io.InputStream;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import shohaku.composer.Composer;
import shohaku.composer.CompositeException;
import shohaku.composer.CompositeRule;
import shohaku.composer.Node;

/**
 * XMLで定義されたプロパティを読み込み保存する機能を提供します。 <br>
 * 
 * <p>
 * 例：
 * 
 * <pre>
 * &lt;xml-properties&gt;
 *     &lt;content name="foo"&gt;
 *          &lt;byte name="byte"&gt;-123&lt;/byte&gt;
 *          &lt;double name="double"&gt;+1,521,414.012411&lt;/double&gt;
 *          &lt;int name="int"&gt;100&lt;/int&gt;
 *          &lt;string name="string"&gt;文字列&lt;/string&gt;
 *          &lt;list id="season" desc="四季"&gt;
 *              &lt;string&gt;春&lt;/ref&gt;
 *              &lt;string&gt;夏&lt;/ref&gt;
 *              &lt;string&gt;秋&lt;/ref&gt;
 *              &lt;string&gt;冬&lt;/ref&gt;
 *          &lt;/list&gt;
 *     &lt;/content&gt;
 * &lt;/xml-properties&gt;
 * </pre>
 * 
 * は以下の様に '/' で区切られた名前階層パスにマッピングされ生成されます。 <br>
 * 
 * <pre>
 * foo/byte   = -123
 * foo/double = 1521414.012411
 * foo/int    = 100
 * foo/string = "文字列"
 * foo/season = ["春", "夏", "秋", "冬"]
 * </pre>
 * 
 * 有効な階層の深さは定義ファイルで変更出来ますが、デフォルトは４階層です。
 * 
 */
public class XMLProperties implements ConfigurationResources, ConfigurationResourcesLoader {

    /** プロパティを格納します。 */
    private final Map lookup = new LinkedHashMap();

    /** プロパティを読取り専用で格納します。 */
    private final Map unmodifiableLookup = Collections.unmodifiableMap(lookup);

    /** Composer。 */
    private final Composer composer = new Composer();

    /** 解析処理に使用するClassLoader. */
    private ClassLoader classLoader;

    /**
     * デフォルトコンストラクタ。
     */
    public XMLProperties() {
        super();
    }

    /**
     * 解析処理に使用する ClassLoader を返します.
     * 
     * @return 解析処理に使用する ClassLoader
     */
    public ClassLoader getClassLoader() {
        return this.classLoader;
    }

    /**
     * 解析処理に使用する ClassLoader を設定します.
     * 
     * @param classLoader
     *            解析処理に使用する ClassLoader
     */
    public void setClassLoader(ClassLoader classLoader) {
        this.classLoader = classLoader;

    }

    /**
     * 入力ストリームからプロパティリストを読み込む。
     * 
     * @param inStream
     *            プロパティの入力ストリーム
     * @throws CompositeException
     *             構成情報例外
     */
    public synchronized void load(InputStream inStream) throws CompositeException {
        load(inStream, getDefaultComposeRule());
    }

    /**
     * 入力ストリームからプロパティリストを読み込む。
     * 
     * @param inStream
     *            プロパティの入力ストリーム
     * @param rule
     *            構成ルール
     * @throws CompositeException
     *             構成情報例外
     */
    public synchronized void load(InputStream inStream, CompositeRule rule) throws CompositeException {
        //XMLの解析
        composer.parse(rule, inStream);
        Node root = composer.getRoot();
        initValues(root);
    }

    /* プロパティの初期化および登録をします。 */
    private void initValues(Node root) {
        if (isPersistent(root)) {
            Iterator i = root.getNodeContext().elementIterator("content");
            while (i.hasNext()) {
                Node node = (Node) i.next();
                initValues(node, node);
            }
        }
    }

    /* 再帰的にノードを読み取り値を登録します。 */
    private void initValues(Node root, Node node) {

        //子のノードを全登録
        Iterator i = node.getNodeContext().elementIterator();
        while (i.hasNext()) {
            Node n = (Node) i.next();

            if (isPersistent(n)) {

                if (n.isType(Node.TYPE_CONTAINER)) {
                    //再起呼出
                    initValues(root, n);

                } else if (n.isType(Node.TYPE_VALUE)) {
                    String path = getPath(root, node, n);
                    if (path != null) {
                        saveValue(path, n);
                    }
                }
            }

        }
    }

    private void saveValue(Object path, Node node) {
        this.lookup.put(path, node.getNodeValue());
    }

    /* この情報のアドレスを返す。 */
    private String getPath(Node root, Node parent, Node node) {

        //永続化しない情報はNullを返す。
        if (!isPersistent(node)) {
            return null;
        }
        //名前階層へ参加する情報は親情報から再帰処理で構築する
        StringBuffer sb = new StringBuffer(80);
        Node c = node;
        while (c != parent && c != null) {
            String name = getNodeName(c);
            if (name == null || name.length() == 0) {
                return null;
            }
            sb.insert(0, name);
            sb.insert(0, '/');
            c = c.getNodeContext().getParent();
        }
        if (node.isType(Node.TYPE_CONTAINER)) {
            sb.append('/');
        }
        if (root != null) {
            String name = getNodeName(root);
            if (name != null && name.length() != 0) {
                sb.insert(0, name);
                sb.insert(0, '/');
            }
        }
        return sb.toString();
    }

    /* 永続化するノードの場合Trueを返却します。 */
    private boolean isPersistent(Node node) {
        return (Node.SCOPE_PRIVATE > node.getScope());
    }

    /* ノード名を返却します。 */
    private String getNodeName(Node node) {
        String name = null;
        if (!node.isType(Node.TYPE_TEXT)) {
            name = node.getAttribute("name");
            if (name == null || name.length() == 0) {
                name = null;
            }
        } else {
            name = "#TEXT#";
        }
        return name;
    }

    /*
     * utils
     */

    /**
     * 全てのプロパティをMapに格納して返却します。 <br>
     * 読取専用Mapで返却します。
     * 
     * @return 全てのプロパティ
     */
    public Map toMap() {
        return unmodifiableLookup;
    }

    /**
     * 全てのプロパティ名を返す。
     * 
     * @return 全てのプロパティ名
     */
    public Set getNames() {
        return unmodifiableLookup.keySet();
    }

    /*
     * implements ConfigurationResources
     */

    /**
     * 全てのプロパティキーを含む反復子を返却します。
     * 
     * @return 全てのプロパティキーを含む反復子
     */
    public Iterator keyIterator() {
        return getNames().iterator();
    }

    /**
     * プロパティキーが示す値を返却します。 指定されたキーが存在しない場合Nullを返却します。
     * 
     * @param key
     *            プロパティ名
     * @return 指定された値
     * @throws NullPointerException
     *             key が null の場合発生する
     */
    public Object getObject(String key) {
        if (key == null) {
            throw new NullPointerException("resource key as null");
        }
        Object obj = lookup.get(key);
        return obj;
    }

    /**
     * プロパティキーが示す値を返却します。 指定されたキーが存在しない場合 defaultValue を返却します。
     * 
     * @param key
     *            プロパティキー
     * @param defaultValue
     *            プロパティキー
     * @return プロパティキーが示す値
     * @throws NullPointerException
     *             key が null の場合発生する
     */
    public Object getObject(String key, Object defaultValue) {
        Object o = getObject(key);
        if (o == null) {
            o = defaultValue;
        }
        return o;
    }

    /**
     * 指定されたキーがプロパティセットに含まれている場合に true を返す。
     * 
     * @param key
     *            プロパティキー
     * @return 指定されたキーが含まれている場合 true
     * @throws NullPointerException
     *             key が null の場合発生する
     */
    public boolean containsKey(String key) {
        if (key == null) {
            throw new NullPointerException("resource key as null");
        }
        return lookup.containsKey(key);
    }

    /**
     * 指定されたキーがプロパティセットに含まれていると同時に、 指定されたクラスとキャスト可能な関係に有る場合に true を返す。 値がNullの場合キャスト可能(True)を返却します。
     * 
     * @param key
     *            プロパティキー
     * @param type
     *            キャスト可能な関係に有るか検証するクラス
     * @return 指定されたキーが含まれている場合 true
     * @throws NullPointerException
     *             key または type が null の場合発生する
     */
    public boolean containsKey(String key, Class type) {
        if (key == null || type == null) {
            throw new NullPointerException("resource key or type as null");
        }
        Object o = getObject(key);
        if (o != null) {
            return type.isInstance(o);
        } else {
            return containsKey(key);
        }
    }

    /*
     * static
     */

    /**
     * デフォルトの構成ルールを返却します。
     * 
     * @return デフォルトの構成ルール
     */
    public static CompositeRule getDefaultComposeRule() {
        return ConfigUtils.getDefaultComposeRule(XMLProperties.class);
    }

}
